from datetime import datetime
import json
# First, try to import modern http.client and support also old one on EPEL 7/Python 2
try:
    import http.client as httplib
except ImportError:
    import httplib

from mock import patch

from blockerbugs import db
from blockerbugs import app
from blockerbugs.models.build import Build
from blockerbugs.models.userinfo import UserInfo
from testing.test_controllers import add_release, add_milestone, \
    add_bug, add_update
from blockerbugs.controllers.api import errors


class TestRestAPI(object):
    @classmethod
    def setup_class(cls):
        db.session.rollback()
        db.drop_all()
        db.create_all()
        cls.client = app.test_client()
        cls.release = add_release(99)
        cls.milestone = add_milestone(cls.release, 'final', 100, 101,
                                      '99-final', True)
        cls.milestone.current = True

        cls.milestone2 = add_milestone(cls.release, 'beta', 200, 201,
                                       '99-beta')
        bug1 = add_bug(9000, 'testbug1', cls.milestone)
        bug1.accepted_fe = True
        bug2 = add_bug(9002, 'testbug2', cls.milestone)
        bug2.accepted_blocker = False
        bug2.proposed_fe = True
        cls.update_pending_stable = add_update(u'test-pending-stable.fc99',
                                               u'stable', [bug1], cls.release,
                                               [cls.milestone, cls.milestone2])
        cls.update_pending_stable.date_submitted = datetime(1990, 1, 1)
        cls.update_pending_stable.pending = True
        cls.update_testing2 = add_update(u'test-testing2.fc99', u'testing',
                                         [bug2],
                                         cls.release, [cls.milestone])
        build = Build()
        build.koji_id = 33
        build.nvr = 'libofx-0.9.9-1.fc20'
        cls.api_user = UserInfo('api_user')
        passwd = cls.api_user.generate_api_key()
        cls.headers = {'X-Auth-User': 'api_user', 'X-Auth-Key': passwd}
        db.session.add(cls.api_user)
        db.session.commit()

    @classmethod
    def teardown_class(cls):
        db.session.rollback()
        db.drop_all()

    def test_unknown_release(self):
        url = '/api/v0/milestones/9999/unknown/bugs'
        resp = self.client.get(url)
        assert resp.status_code == httplib.NOT_FOUND
        error = json.loads(resp.data)['error']
        assert error['code'] == errors.NoSuchObjectError.code
        assert 'Release' in error['message']

    def test_unknown_milestone(self):
        url = '/api/v0/milestones/99/unknown/bugs'
        resp = self.client.get(url)
        assert resp.status_code == httplib.NOT_FOUND
        error = json.loads(resp.data)['error']
        assert error['code'] == errors.NoSuchObjectError.code
        assert 'Milestone' in error['message']

    def test_list_all_bugs(self):
        url = '/api/v0/milestones/99/final/bugs'
        resp = self.client.get(url)
        assert resp.status_code == httplib.OK
        data = json.loads(resp.data)
        assert len(data) == 2
        bug = data[0]
        assert bug['bugid'] == 9000
        assert bug['url'] == 'https://bugzilla.redhat.com/show_bug.cgi?id=9000'
        assert bug['summary'] == 'testbug1'
        assert bug['component'] == 'testcomponent'
        assert bug['active']
        assert set(bug['type']) == set(('accepted_blocker', 'accepted_fe'))

    def test_list_accepted_blocker_bugs(self):
        url = '/api/v0/milestones/99/final/bugs?bugtype=accepted_blocker&'
        resp = self.client.get(url)
        assert resp.status_code == httplib.OK
        data = json.loads(resp.data)
        assert len(data) == 1
        assert data[0]['bugid'] == 9000

    def test_list_all_updates(self):
        url = '/api/v0/milestones/99/final/updates'
        resp = self.client.get(url)
        assert resp.status_code == httplib.OK
        data = json.loads(resp.data)
        assert len(data) == 2
        update = data[-1]
        assert update['title'] == u'test-pending-stable.fc99'
        assert update['url'] == u'http://localhost/update'
        assert update['karma'] == 1
        assert update['stable_karma'] == 3
        assert update['status'] == u'stable'
        assert update['pending']
        assert update['bugs'][0]['bugid'] == 9000
        assert set(update['bugs'][0]['type']) == set(('accepted_blocker',
                                                  'accepted_fe'))
        assert update['release'] == 99
        assert len(update['milestones']) == 2
        assert {'version': 'final', 'release': 99} in update['milestones']
        assert {'version': 'beta', 'release': 99} in update['milestones']

    def test_list_updates_with_proposed_fe_bugs(self):
        url = '/api/v0/milestones/99/final/updates?bugtype=proposed_fe&'
        resp = self.client.get(url)
        assert resp.status_code == httplib.OK
        data = json.loads(resp.data)
        assert len(data) == 1
        update = data[0]
        assert update['title'] == u'test-testing2.fc99'

    def test_bad_bugtype_list_bugs(self):
        url = '/api/v0/milestones/99/final/updates?bugtype=foo&'
        resp = self.client.get(url)
        assert resp.status_code == httplib.BAD_REQUEST
        error = json.loads(resp.data)['error']
        assert error['code'] == errors.InvalidArgumentError.code
        assert 'bugtype' in error['message']

    def test_get_current_milestone(self):
        url = '/api/v0/milestones/current'
        resp = self.client.get(url)
        assert resp.status_code == httplib.OK
        data = json.loads(resp.data)
        assert data['name'] == self.milestone.name
