import datetime

from blockerbugs.models.milestone import Milestone
from blockerbugs.models.release import Release
from blockerbugs.models.bug import Bug
from blockerbugs.models.update import Update
from blockerbugs.models.criterion import Criterion
from blockerbugs import db
from blockerbugs import app
from blockerbugs.controllers import main


def add_release(number=99):
    test_release = Release(number)
    db.session.add(test_release)
    db.session.commit()
    return test_release


def add_milestone(release, version, blocker_tracker, fe_tracker, name,
                  active=False):
    test_milestone = Milestone(release, version, blocker_tracker, fe_tracker,
                               name, active)
    db.session.add(test_milestone)
    db.session.commit()
    return test_milestone


def add_bug(bugid, summary, milestone):
    test_bug = Bug(bugid,
                   'https://bugzilla.redhat.com/show_bug.cgi?id=%d' % bugid,
                   summary, 'NEW', 'testcomponent', milestone, True, True, 'John Doe')
    test_bug.accepted_blocker = True
    db.session.add(test_bug)
    db.session.commit()
    return test_bug


def add_update(title, status, bugs, release, milestones,
               url='http://localhost/update'):
    test_update = Update(title, url, 1, status, bugs,
                         release, milestones,
                         date_submitted=datetime.datetime.utcnow())
    test_update.stable_karma = 3
    db.session.add(test_update)
    db.session.commit()
    return test_update


class TestEmptyDB(object):
    @classmethod
    def setup_class(cls):
        cls.client = app.test_client()
        db.session.rollback()
        db.drop_all()
        db.create_all()

    @classmethod
    def teardown_class(cls):
        db.session.rollback()
        db.drop_all()

    def test_index_returns_200code(self):
        rv = self.client.get('/')
        assert rv.status_code == 200


class TestNotSyncedMilestone(object):
    @classmethod
    def setup_class(cls):
        cls.client = app.test_client()
        db.session.rollback()
        db.drop_all()
        db.create_all()
        release = add_release(99)
        final_milestone = add_milestone(release, 'final', 100, 101,
                                        '99-final', True)
        add_milestone(release, 'beta', 200, 201, '99-beta')

    @classmethod
    def teardown_class(cls):
        db.session.rollback()
        db.drop_all()

    def test_not_synced_index(self):
        rv = self.client.get('/')
        assert rv.status_code == 200
        assert b'F99 final' in rv.data
        assert b'F99 beta' not in rv.data

    def test_display_buglist_returns_200code(self):
        rv = self.client.get('milestone/99/final/buglist')
        assert rv.status_code == 200

    def test_display_updates_returns_200code(self):
        rv = self.client.get('milestone/99/final/updates')
        assert rv.status_code == 200

    def test_display_blockers_returns_200code(self):
        rv = self.client.get('milestone/99/final/irc')
        assert rv.status_code == 200

    def test_display_info_returns_200code(self):
        rv = self.client.get('milestone/99/final/info')
        assert rv.status_code == 200


class TestSyncedMilestone(object):
    @classmethod
    def setup_class(cls):
        cls.client = app.test_client()
        db.session.rollback()
        db.drop_all()
        db.create_all()
        release = add_release(99)
        final_milestone = add_milestone(release, 'final', 100, 101,
                                        '99-final', True)
        add_milestone(release, 'beta', 200, 201, '99-beta')
        add_bug(9000, 'testbug1', final_milestone)
        add_bug(9001, 'testbug2', final_milestone)
        final_milestone.last_bug_sync = datetime.datetime.utcnow()
        db.session.commit()

    @classmethod
    def teardown_class(cls):
        db.session.rollback()
        db.drop_all()

    def test_display_index(self):
        rv = self.client.get('/')
        assert rv.status_code == 200
        assert b'F99 final' in rv.data
        assert b'F99 beta' not in rv.data

    def test_display_milestone_redirect(self):
        rv = self.client.get('milestone/99/final')
        assert rv.status_code == 302

    def test_display_buglist(self):
        rv = self.client.get('milestone/99/final/buglist')
        assert rv.status_code == 200
        assert b'9000' in rv.data
        assert b'testbug2' in rv.data

    def test_display_updates_returns_200code(self):
        rv = self.client.get('milestone/99/final/updates')
        assert rv.status_code == 200

    def test_display_irc_blockers(self):
        rv = self.client.get('milestone/99/final/irc')
        assert rv.status_code == 200
        assert b'9000' in rv.data
        assert b'testbug2' in rv.data

    def test_display_info(self):
        rv = self.client.get('milestone/99/final/info')
        assert rv.status_code == 200
        assert b'100' in rv.data
        assert b'101' in rv.data

    def test_display_stats_returns_200code(self):
        rv = self.client.get('milestone/99/final/stats')
        assert rv.status_code == 200


class TestGetFunctions(object):
    @classmethod
    def setup_class(cls):
        db.session.rollback()
        db.drop_all()
        db.create_all()
        cls.release = add_release(99)
        cls.milestone = add_milestone(cls.release, 'final', 100, 101,
                                        '99-final', True)
        add_milestone(cls.release, 'beta', 200, 201, '99-beta')
        bug1 = add_bug(9000, 'testbug1', cls.milestone)
        bug2 = add_bug(9001, 'testbug2', cls.milestone)
        bug2.accepted_blocker = False
        cls.update_pending_stable = add_update(u'test-pending-stable.fc99', u'stable', [bug1],
                            cls.release, [cls.milestone])
        cls.update_pending_stable.pending = True
        cls.update_stable = add_update(u'test-stable1.fc99', u'stable', [bug1],
                             cls.release, [cls.milestone])
        cls.update_stable = add_update(u'test-stable2.fc99', u'stable', [bug2],
                                       cls.release, [cls.milestone])
        cls.update_pending_testing = add_update(u'test-pending-testing.fc99', u'testing', [bug1],
                            cls.release, [cls.milestone])
        cls.update_pending_testing.pending = True
        cls.update_testing1 = add_update(u'test-testing1.fc99', u'testing', [bug1],
                                        cls.release, [cls.milestone])
        cls.update_testing2 = add_update(u'test-testing2.fc99', u'testing', [bug2],
                                         cls.release, [cls.milestone])
        db.session.commit()

    @classmethod
    def teardown_class(cls):
        db.session.rollback()
        db.drop_all()

    def test_get_pending_stable_updates(self):
        updates = main.get_milestone_pending_stable_updates(self.milestone)
        assert len(updates) == 1
        update = updates[0]
        assert self.update_pending_stable == update


    def test_get_testing_status_and_has_bugs_updates(self):
        updates = main.get_milestone_updates_testing(self.milestone)
        assert len(updates) == 1
        update = updates[0]
        assert self.update_testing1 == update

    def test_get_milestone_all_nonstable_blocker_fixes(self):
        updates = main.get_milestone_all_nonstable_blocker_fixes(self.milestone)
        assert len(updates) == 3
        expected = [self.update_pending_stable, self.update_pending_testing, self.update_testing1]
        updates.sort(key=lambda x: x.title)
        expected.sort(key=lambda x: x.title)
        assert updates == expected

    def test_get_milestone_all_nonstable_fe_fixes(self):
        bug3 = add_bug(9002, 'testbug3', self.milestone)
        bug3.accepted_blocker = False
        bug3.accepted_fe = True
        update_testing3 = add_update(u'test-testing3.fc99', u'testing', [bug3],
                                     self.release, [self.milestone])
        updates = main.get_milestone_all_nonstable_fe_fixes(self.milestone)
        assert len(updates) == 1
        update = updates[0]
        assert update_testing3 == update
